@@ -15,25 +15,75 @@
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
 #include <netdb.h>
+#include <stdlib.h>
 
+#include "config.h"
 #include "config-srcpd.h"
 #include "srcp-fb.h"
 #include "srcp-power.h"
 #include "srcp-server.h"
+#include "srcp-error.h"
+
+#ifdef USE_DDL
 #include "ddl.h"
-#include "m605x.h"
-#include "selectrix.h"
-#include "ib.h"
-#include "loopback.h"
+#endif
+
+#ifdef USE_DDLS88
 #include "ddl-s88.h"
+#endif
+
+#ifdef USE_HSI88
 #include "hsi-88.h"
+#endif
+
+#ifdef USE_I2C
 #include "i2c-dev.h"
-#include "zimo.h"
+#endif
+
+#ifdef USE_IB
+#include "ib.h"
+#endif
+
+#ifdef USE_LI100
 #include "li100.h"
+#endif
+
+#ifdef USE_LOCONET
 #include "loconet.h"
+#endif
+
+#ifdef USE_LOOPBACK
+#include "loopback.h"
+#endif
+
+#ifdef USE_M605X
+#include "m605x.h"
+#endif
+
+#ifdef USE_SELECTRIX
+#include "selectrix.h"
+#endif
+
+#ifdef USE_ZIMO
+#include "zimo.h"
+#endif
+
+#ifdef USE_DCCAR
+#include "dccar.h"
+#endif
+
 #include "syslogmessage.h"
 
 
+/* define global variables */
+bus_data_t buses[MAX_BUSES];
+bus_t num_buses = 1;
+
+
+static const char DISABLE_MSG[] =
+    "\"%s\" has been disabled at compile time.\n";
+
+
 /* check if a bus has a device group or not */
 int bus_has_devicegroup(bus_t bus, int dg)
 {
@@ -65,6 +115,26 @@ int bus_has_devicegroup(bus_t bus, int dg)
     return 0;
 }
 
+/**
+  check if a given bus supports a specified protocol
+  @par Input: bus_t busnumber the bus
+  @par Input: const char protocol the protocol
+  @return SRCP_OK i.e. the protocol is found, else SRCP Error code */
+int bus_supports_protocol(bus_t busnumber, const char protocol)
+{
+    size_t i;
+
+    if (buses[busnumber].protocols) {
+        char const *protocols = buses[busnumber].protocols;
+        for (i = 0; i < strlen(protocols); i++) {
+            if (protocols[i] == protocol) {
+                return SRCP_OK;
+            }
+        }
+    }
+    return SRCP_UNSUPPORTEDDEVICEPROTOCOL;
+}
+
 static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
 {
     int result;
@@ -74,19 +144,18 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
         return busnumber;
 
     if (busnumber >= MAX_BUSES) {
-            syslog_bus(0, DBG_ERROR,
-               "Sorry, you have used an invalid bus number (%ld). "
-               "If this is greater than or equal to %d,\n"
-               "you need to recompile the sources.\n",
-               busnumber, MAX_BUSES);
+        syslog_bus(0, DBG_ERROR,
+                   "Sorry, you have used an invalid bus number (%ld). "
+                   "If this is greater than or equal to %d,\n"
+                   "you need to recompile the sources.\n",
+                   busnumber, MAX_BUSES);
         return busnumber;
     }
 
-    num_buses = busnumber;
-
     /* some default values */
     buses[current_bus].debuglevel = DBG_INFO;
     buses[current_bus].flags = 0;
+    buses[current_bus].protocols = 0;
 
     /* Function pointers to NULL */
     buses[current_bus].thr_func = NULL;
@@ -102,19 +171,19 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
     buses[current_bus].device.file.path = NULL;
 
     /* Definition of thread synchronisation  */
-    /*TODO: this should be (privately) moved to each bus*/
+    /*TODO: this should be (privately) moved to each bus */
     result = pthread_mutex_init(&buses[current_bus].transmit_mutex, NULL);
     if (result != 0) {
         syslog_bus(current_bus, DBG_ERROR,
-                "pthread_mutex_init() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_init() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     result = pthread_cond_init(&buses[current_bus].transmit_cond, NULL);
     if (result != 0) {
         syslog_bus(current_bus, DBG_ERROR,
-                "pthread_cond_init() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_cond_init() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     xmlNodePtr child = node->children;
@@ -123,8 +192,8 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
 
     while (child != NULL) {
 
-        if ((xmlStrcmp(child->name, BAD_CAST "text") == 0) ||
-            (xmlStrcmp(child->name, BAD_CAST "comment") == 0)) {
+        if ((xmlStrncmp(child->name, BAD_CAST "text", 4) == 0) ||
+            (xmlStrncmp(child->name, BAD_CAST "comment", 7) == 0)) {
             /* just do nothing, it is only formatting text or a comment */
         }
 
@@ -132,69 +201,125 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
             if (busnumber == 0)
                 busnumber += readconfig_server(doc, child, busnumber);
             else
-                syslog_bus(0, DBG_ERROR, "Sorry, type=server is not allowed "
-                                "at bus %ld!\n", busnumber);
-        }
-
-        /* but the most important are not ;=)  */
-        else if (xmlStrcmp(child->name, BAD_CAST "zimo") == 0) {
-            busnumber += readconfig_ZIMO(doc, child, busnumber);
+                syslog_bus(0, DBG_ERROR,
+                           "Sorry, type=server is not allowed "
+                           "at bus %ld!\n", busnumber);
         }
 
         else if (xmlStrcmp(child->name, BAD_CAST "ddl") == 0) {
+#ifdef USE_DDL
             busnumber += readconfig_DDL(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "m605x") == 0) {
-            busnumber += readconfig_m605x(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "intellibox") == 0) {
-            busnumber += readConfig_IB(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "loconet") == 0) {
-            busnumber += readConfig_LOCONET(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "loopback") == 0) {
-            busnumber += readconfig_LOOPBACK(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
         }
 
         else if (xmlStrcmp(child->name, BAD_CAST "ddl-s88") == 0) {
+#ifdef USE_DDLS88
 #if defined(linux) || defined(__CYGWIN__) || defined(__FreeBSD__)
             busnumber += readconfig_DDL_S88(doc, child, busnumber);
 #else
-            syslog_bus(0, DBG_ERROR, "Sorry, DDL-S88 not (yet) available on "
-                            "this system.\n");
+            syslog_bus(0, DBG_ERROR,
+                       "Sorry, DDL-S88 not (yet) available on "
+                       "this system.\n");
+#endif
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
 #endif
         }
 
         else if (xmlStrcmp(child->name, BAD_CAST "hsi-88") == 0) {
+#ifdef USE_HSI88
             busnumber += readConfig_HSI_88(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "li100usb") == 0) {
-            busnumber += readConfig_LI100_USB(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "li100") == 0) {
-            busnumber += readConfig_LI100_SERIAL(doc, child, busnumber);
-        }
-
-        else if (xmlStrcmp(child->name, BAD_CAST "selectrix") == 0) {
-            busnumber += readconfig_Selectrix(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
         }
 
         else if (xmlStrcmp(child->name, BAD_CAST "i2c-dev") == 0) {
+#ifdef USE_I2C
 #ifdef linux
             busnumber += readconfig_I2C_DEV(doc, child, busnumber);
 #else
             syslog_bus(0, DBG_ERROR, "Sorry, I2C-DEV is only available on "
-                            "Linux (yet).\n");
+                       "Linux (yet).\n");
+#endif
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
 #endif
         }
 
+        else if (xmlStrcmp(child->name, BAD_CAST "intellibox") == 0) {
+#ifdef USE_IB
+            busnumber += readConfig_IB(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "li100usb") == 0) {
+#ifdef USE_LI100
+            busnumber += readConfig_LI100_USB(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "li100") == 0) {
+#ifdef USE_LI100
+            busnumber += readConfig_LI100_SERIAL(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "loconet") == 0) {
+#ifdef USE_LOCONET
+            busnumber += readConfig_LOCONET(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "loopback") == 0) {
+#ifdef USE_LOOPBACK
+            busnumber += readconfig_LOOPBACK(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "m605x") == 0) {
+#ifdef USE_M605X
+            busnumber += readconfig_m605x(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+        else if (xmlStrcmp(child->name, BAD_CAST "selectrix") == 0) {
+#ifdef USE_SELECTRIX
+            busnumber += readconfig_Selectrix(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+        else if (xmlStrcmp(child->name, BAD_CAST "zimo") == 0) {
+#ifdef USE_ZIMO
+            busnumber += readconfig_ZIMO(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+        else if (xmlStrcmp(child->name, BAD_CAST "dccar") == 0) {
+#ifdef USE_DCCAR
+            busnumber += readconfig_DCCAR(doc, child, busnumber);
+#else
+            syslog_bus(0, DBG_ERROR, DISABLE_MSG, child->name);
+#endif
+        }
+
+
         /* some attributes are common for all (real) buses */
         else if (xmlStrcmp(child->name, BAD_CAST "device") == 0) {
             txt2 = xmlGetProp(child, BAD_CAST "type");
@@ -205,8 +330,9 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
                 buses[current_bus].devicetype = HW_NETWORK;
             }
             else {
-                syslog_bus(0, DBG_ERROR, "WARNING, \"%s\" (bus %ld) is an "
-                     "unknown device specifier!\n", child->name, current_bus);
+                syslog_bus(0, DBG_WARN, "WARNING, \"%s\" (bus %ld) is an "
+                           "unknown device property!\n", txt2,
+                           current_bus);
             }
             free(txt2);
             txt = xmlNodeListGetString(doc, child->children, 1);
@@ -218,6 +344,21 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
                             malloc(strlen((char *) txt) + 1);
                         strcpy(buses[current_bus].device.file.path,
                                (char *) txt);
+                        txt2 = xmlGetProp(child, BAD_CAST "flowcontrol");
+                        if(txt2!=NULL) {
+                          if(xmlStrcmp(txt2, BAD_CAST "none")==0) {
+                             buses[current_bus].device.file.settings = SER_FC_NONE;
+                          }else if(xmlStrcmp(txt2, BAD_CAST "hard")==0) {
+                             buses[current_bus].device.file.settings = SER_FC_HARD;
+                          }else if(xmlStrcmp(txt2, BAD_CAST "soft")==0) {
+                             buses[current_bus].device.file.settings = SER_FC_SOFT;
+                          }else {
+                             syslog_bus(0, DBG_WARN, "WARNING, \"%s\" (bus %ld) is an unknown flow control option "
+                               "unknown device property!\n", txt2, current_bus);
+                          }
+                          free(txt2);
+                        }
+
                         break;
                     case HW_NETWORK:
                         free(buses[current_bus].device.file.path);
@@ -227,18 +368,22 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
                                (char *) txt);
                         txt2 = xmlGetProp(child, BAD_CAST "port");
                         if (txt2 != NULL) {
+                            free(buses[current_bus].device.net.port);
                             buses[current_bus].device.net.port =
-                                atoi((char *) txt2);
+                                malloc(strlen((char *) txt2) + 1);
+                            strcpy(buses[current_bus].device.net.port,
+                                   (char *) txt2);
                             free(txt2);
                         }
                         else {
-                            buses[current_bus].device.net.port = 0;
+                            buses[current_bus].device.net.port = NULL;
                         }
                         txt2 = xmlGetProp(child, BAD_CAST "protocol");
                         if (txt2 != NULL) {
                             struct protoent *p;
                             p = getprotobyname((char *) txt2);
-                            buses[current_bus].device.net.protocol = p->p_proto;
+                            buses[current_bus].device.net.protocol =
+                                p->p_proto;
                             free(txt2);
                         }
                         else {
@@ -251,14 +396,14 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
             switch (buses[current_bus].devicetype) {
                 case HW_FILENAME:
                     syslog_bus(current_bus, DBG_DEBUG, "** Filename='%s'",
-                        buses[current_bus].device.file.path);
+                               buses[current_bus].device.file.path);
                     break;
                 case HW_NETWORK:
                     syslog_bus(current_bus, DBG_DEBUG,
-                        "** Network Host='%s', Protocol=%d Port=%d",
-                        buses[current_bus].device.net.hostname,
-                        buses[current_bus].device.net.protocol,
-                        buses[current_bus].device.net.port);
+                               "** Network Host='%s', Protocol=%d Port=%s",
+                               buses[current_bus].device.net.hostname,
+                               buses[current_bus].device.net.protocol,
+                               buses[current_bus].device.net.port);
                     break;
             }
         }
@@ -266,7 +411,14 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
         else if (xmlStrcmp(child->name, BAD_CAST "verbosity") == 0) {
             txt = xmlNodeListGetString(doc, child->children, 1);
             if (txt != NULL) {
-                buses[current_bus].debuglevel = atoi((char *) txt);
+                int verbosity = atoi((char *) txt);
+                if ((verbosity < 0) || (verbosity > DBG_DEBUG)) {
+                    syslog_bus(current_bus, DBG_ERROR,
+                               "Invalid verbosity value found: %d; using %d instead",
+                               verbosity, DBG_DEBUG);
+                    verbosity = DBG_DEBUG;
+                }
+                buses[current_bus].debuglevel = verbosity;
                 xmlFree(txt);
             }
         }
@@ -334,14 +486,27 @@ static bus_t register_bus(bus_t busnumber, xmlDocPtr doc, xmlNodePtr node)
             }
         }
 
+        else if (xmlStrcmp(child->name, BAD_CAST "auto_speed_detection") ==
+                 0) {
+            txt = xmlNodeListGetString(doc, child->children, 1);
+            if (txt != NULL) {
+                if (xmlStrcmp(txt, BAD_CAST "yes") == 0)
+                    buses[current_bus].flags |= USE_AUTODETECTION;
+                else if (xmlStrcmp(txt, BAD_CAST "no") == 0)
+                    buses[current_bus].flags &= ~USE_AUTODETECTION;
+                xmlFree(txt);
+                syslog_bus(0, DBG_ERROR,
+                        "Setting speed_detection for bus: %ld\n", current_bus);
+            }
+        }
+
         else
             syslog_bus(0, DBG_ERROR,
-                    "WARNING, \"%s\" (bus %ld) is an unknown tag!\n",
-                    child->name, current_bus);
+                       "WARNING, \"%s\" (bus %ld) is an unknown tag!\n",
+                       child->name, current_bus);
 
         child = child->next;
     }
-
     return busnumber;
 }
 
@@ -360,6 +525,7 @@ static bus_t walk_config_xml(xmlDocPtr doc)
 
     while (child != NULL) {
         bus = register_bus(bus, doc, child);
+        num_buses = bus - 1;
         child = child->next;
     }
     return bus;
@@ -376,14 +542,14 @@ int readConfig(char *filename)
     num_buses = 0;
 
     /* some defaults */
-    syslog_bus(0, DBG_DEBUG, "parsing %s", filename);
+    syslog_bus(0, DBG_DEBUG, "Parsing %s", filename);
     doc = xmlParseFile(filename);
 
     /* always show a message */
     if (doc != NULL) {
-        syslog_bus(0, DBG_DEBUG, "walking %s", filename);
+        syslog_bus(0, DBG_DEBUG, "Walking %s", filename);
         rb = walk_config_xml(doc);
-        syslog_bus(0, DBG_DEBUG, " done %s; found %ld buses", filename, rb);
+        syslog_bus(0, DBG_DEBUG, "Done %s; found %ld buses", filename, rb);
         xmlFreeDoc(doc);
         /*
          *Free the global variables that may
@@ -393,7 +559,8 @@ int readConfig(char *filename)
     }
     else {
         syslog_bus(0, DBG_ERROR,
-            "Error, no XML document tree found parsing %s.\n", filename);
+                   "Error, no XML document tree found parsing %s.\n",
+                   filename);
     }
     return (rb > 0);
 }
@@ -414,24 +581,24 @@ void suspend_bus_thread(bus_t busnumber)
     result = pthread_mutex_lock(&buses[busnumber].transmit_mutex);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_mutex_lock() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_lock() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     result = pthread_cond_wait(&buses[busnumber].transmit_cond,
-            &buses[busnumber].transmit_mutex);
+                               &buses[busnumber].transmit_mutex);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_cond_wait() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_cond_wait() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
-    
+
     /* mutex released.       */
     result = pthread_mutex_unlock(&buses[busnumber].transmit_mutex);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_mutex_unlock() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_unlock() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
     syslog_bus(busnumber, DBG_DEBUG, "Bus thread is working again.");
 }
@@ -448,22 +615,22 @@ void resume_bus_thread(bus_t busnumber)
     result = pthread_mutex_lock(&buses[busnumber].transmit_mutex);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_mutex_lock() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_lock() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     result = pthread_cond_signal(&buses[busnumber].transmit_cond);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_cond_signal() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_cond_signal() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
-    
+
     result = pthread_mutex_unlock(&buses[busnumber].transmit_mutex);
     if (result != 0) {
         syslog_bus(busnumber, DBG_ERROR,
-                "pthread_mutex_lock() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_lock() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
     syslog_bus(0, DBG_DEBUG, "Thread on bus %d is woken up", busnumber);
 }
@@ -476,41 +643,41 @@ void create_all_bus_threads()
     bus_t i;
 
     syslog_bus(0, DBG_INFO, "Starting %ld bus interface threads.",
-            num_buses);
+               num_buses);
 
     /* start threads for all buses */
     for (i = 1; i <= num_buses; i++) {
         syslog_bus(0, DBG_INFO,
-                "Starting interface thread number %ld (type = %d).",
-                i, buses[i].type);
+                   "Starting interface thread number %ld (type = %d).",
+                   i, buses[i].type);
 
         if (buses[i].thr_timer != NULL) {
-               result = pthread_create(&ttid_tid, NULL, buses[i].thr_timer,
-                                        (void *) i);
-               if (result != 0) {
-                   syslog(LOG_INFO, "Create timer thread for bus %ld "
-                           "failed: %s (errno = %d)\n", i,
-                           strerror(result), result);
-                   exit(1);
-               }
-               buses[i].tidtimer = ttid_tid;
+            result = pthread_create(&ttid_tid, NULL, buses[i].thr_timer,
+                                    (void *) i);
+            if (result != 0) {
+                syslog(LOG_INFO, "Create timer thread for bus %ld "
+                       "failed: %s (errno = %d)\n", i,
+                       strerror(result), result);
+                exit(EXIT_FAILURE);
+            }
+            buses[i].tidtimer = ttid_tid;
         }
 
         if (buses[i].thr_func != NULL) {
-               result = pthread_create(&ttid_tid, NULL, buses[i].thr_func,
-                                        (void *) i);
-               if (result != 0) {
-                   syslog(LOG_INFO, "Create interface thread for bus %ld "
-                           "failed: %s (errno = %d)\n", i,
-                           strerror(result), result);
-                   exit(1);
-               }
-               buses[i].tid = ttid_tid;
+            result = pthread_create(&ttid_tid, NULL, buses[i].thr_func,
+                                    (void *) i);
+            if (result != 0) {
+                syslog(LOG_INFO, "Create interface thread for bus %ld "
+                       "failed: %s (errno = %d)\n", i,
+                       strerror(result), result);
+                exit(EXIT_FAILURE);
+            }
+            buses[i].tid = ttid_tid;
         }
 
         syslog_bus(i, LOG_INFO, "Interface thread started successfully "
-                "(type =%d, tid = %u)", buses[i].type,
-                (unsigned int) (buses[i].tid));
+                   "(type =%d, tid = %u)", buses[i].type,
+                   (unsigned int) (buses[i].tid));
 
         if (((buses[i].flags & AUTO_POWER_ON) == AUTO_POWER_ON)) {
             setPower(i, 1, "AUTO POWER ON");
@@ -526,7 +693,7 @@ void cancel_all_bus_threads()
 {
     bus_t bus;
     int result = 0;
-    void* thr_result;
+    void *thr_result;
 
     for (bus = 1; bus <= num_buses; bus++) {
 
@@ -534,29 +701,29 @@ void cancel_all_bus_threads()
             result = pthread_cancel(buses[bus].tidtimer);
             if (result != 0)
                 syslog_bus(bus, DBG_ERROR,
-                        "Timer thread cancel failed: %s (errno = %d).",
-                        strerror(result), result);
+                           "Timer thread cancel failed: %s (errno = %d).",
+                           strerror(result), result);
 
-            /*wait until timer thread terminates*/
+            /*wait until timer thread terminates */
             result = pthread_join(buses[bus].tidtimer, &thr_result);
             if (result != 0)
                 syslog_bus(bus, DBG_ERROR,
-                        "Timer thread join failed: %s (errno = %d).",
-                        strerror(result), result);
+                           "Timer thread join failed: %s (errno = %d).",
+                           strerror(result), result);
         }
 
         result = pthread_cancel(buses[bus].tid);
         if (result != 0)
             syslog_bus(bus, DBG_ERROR,
-                    "Interface thread cancel failed: %s (errno = %d).",
-                    strerror(result), result);
+                       "Interface thread cancel failed: %s (errno = %d).",
+                       strerror(result), result);
 
-        /*wait until thread terminates*/
+        /*wait until thread terminates */
         result = pthread_join(buses[bus].tid, &thr_result);
         if (result != 0)
             syslog_bus(bus, DBG_ERROR,
-                    "Interface thread join failed: %s (errno = %d).",
-                    strerror(result), result);
+                       "Interface thread join failed: %s (errno = %d).",
+                       strerror(result), result);
 
         syslog_bus(bus, DBG_INFO, "Bus successfully cancelled.");
     }
@@ -571,20 +738,30 @@ void run_bus_watchdog()
 
     for (bus = 1; bus <= num_buses; bus++) {
         if ((buses[bus].flags & USE_WATCHDOG)
-                    && buses[bus].watchdog == 0
-                    && !queue_GL_isempty(bus)
-                    && !queue_GA_isempty(bus)) {
+            && buses[bus].watchdog == 0 && !queue_GL_isempty(bus)
+            && !queue_GA_isempty(bus)) {
             syslog_bus(bus, DBG_ERROR, "Oops: Interface thread "
-                    "hangs, restarting (old tid = %ld, %d).",
-                    (long) buses[bus].tid, buses[bus].watchdog);
-            pthread_cancel(buses[bus].tid);
-            pthread_join(buses[bus].tid, NULL);
+                       "hangs, restarting (old tid = %ld, %d).",
+                       (long) buses[bus].tid, buses[bus].watchdog);
+
+            result = pthread_cancel(buses[bus].tid);
+            if (result != 0)
+                syslog_bus(bus, DBG_ERROR,
+                           "Interface thread cancel failed: %s (errno = %d).",
+                           strerror(result), result);
+
+            result = pthread_join(buses[bus].tid, NULL);
+            if (result != 0)
+                syslog_bus(bus, DBG_ERROR,
+                           "Interface thread join failed: %s (errno = %d).",
+                           strerror(result), result);
+
             result = pthread_create(&ttid_tid, NULL,
-                    buses[bus].thr_func, (void *) bus);
+                                    buses[bus].thr_func, (void *) bus);
             if (result != 0) {
                 syslog(LOG_INFO, "Recreate interface thread "
-                        "failed: %s (errno = %d)\n",
-                        strerror(result), result);
+                       "failed: %s (errno = %d)\n",
+                       strerror(result), result);
                 break;
             }
             buses[bus].tid = ttid_tid;
