@@ -21,6 +21,7 @@
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #ifdef linux
 #include <sys/io.h>
@@ -231,7 +232,7 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
         }
         else
             syslog_bus(busnumber + i, DBG_INFO,
-                       "%d feeback contacts for channel %d successfully "
+                       "%d feedback contacts for channel %d successfully "
                        "initialized.", __ddl_s88->number_fb[i] * 16,
                        i + 1);
     }
