@@ -341,10 +341,10 @@ static char *preamble = "111111111111111";
 static const int NMRA_STACKSIZE = 200;
 
 /* 230 is needed for all functions F0-F28 */
-static const int BUFFERSIZE = 230;
+static const unsigned int BUFFERSIZE = 1000;
 
 /* internal offset of the long addresses */
-static const int ADDR14BIT_OFFSET = 128;
+static const unsigned int ADDR14BIT_OFFSET = 128;
 
 /* the result is only an index, no warranty */
 static int translateabel(char *bs)
@@ -664,7 +664,8 @@ static void calc_28spst_speed_byte(char *byte, int direction, int speed)
         if (speed & 1) {
             byte[3] = '1';
         }
-    } else {
+    }
+    else {
         calc_single_byte(byte, 0x40 | (direction << 5) | speed);
     }
 }
@@ -689,7 +690,8 @@ static void calc_function_group_two_byte(char *byte, int func, int lower)
         func = (func >> 5) & 0xf;
         /* set command for F5 to F8 */
         func |= 0xb0;
-    } else {
+    }
+    else {
         func = (func >> 9) & 0xf;
         func |= 0xa0;
     }
@@ -721,9 +723,10 @@ static void xor_two_bytes(char *byte, char *byte1, char *byte2)
 
 /*** functions to generate NMRA-DCC data packets ***/
 
-int comp_nmra_accessory(bus_t busnumber, int nr, int output, int activate)
+int comp_nmra_accessory(bus_t busnumber, int nr, int output, int activate,
+                        int offset)
 {
-    /* command: NA <nr [0001-4096]> <outp [0,1]> <activate [0,1]>
+    /* command: NA <nr [0001-2044]> <outp [0,1]> <activate [0,1]>
        example: NA 0012 0 1  */
 
     char byte1[9];
@@ -743,7 +746,7 @@ int comp_nmra_accessory(bus_t busnumber, int nr, int output, int activate)
                "(NA) received");
 
     /* no special error handling, it's job of the clients */
-    if (nr < 1 || nr > 4096 || output < 0 || output > 1 ||
+    if (nr < 1 || nr > 2044 || output < 0 || output > 1 ||
         activate < 0 || activate > 1)
         return 1;
 
@@ -752,10 +755,9 @@ int comp_nmra_accessory(bus_t busnumber, int nr, int output, int activate)
 
     /* calculate the real address of the decoder and the pair number 
      * of the switch */
-    /* valid decoder addresses: 0..1023 */
-    address = ((nr - 1) / 4);
-    pairnr = (((nr - 1) % 4) << 1) + output;
-    activate <<= 3;
+    /* valid decoder addresses: 0..511 */
+    address = ((nr - 1) / 4) + offset;
+    pairnr = (nr - 1) % 4;
 
     /* address byte: 10AAAAAA (lower 6 bits) */
     calc_single_byte(byte1, 0x80 | (address & 0x3f));
@@ -763,7 +765,8 @@ int comp_nmra_accessory(bus_t busnumber, int nr, int output, int activate)
     /* address and data 1AAACDDO upper 3 address bits are inverted */
     /* C =  activate, DD = pairnr */
     calc_single_byte(byte2,
-                     0x80 | ((~address) & 0x1c0) >> 2 | activate | pairnr);
+                     0x80 | ((~address) & 0x1c0) >> 2 | activate << 3 |
+                     pairnr << 1 | output);
     xor_two_bytes(byte3, byte2, byte1);
 
     /* putting all together in a 'bitstream' (char array) */
@@ -998,7 +1001,8 @@ static void calc_address_stream(char *addrstream, char *addrerrbyte,
 {
     char addrbyte[9];
     char addrbyte2[9];
-    if (mode == 1) {
+    // if (mode == 1 || mode == 2 || mode == 5) { // Torsten Vogt 2012-01-15
+    if (mode == 1) { // Torsten Vogt 2012-11-10
         /* calc 7 bit address - leading bit is zero */
         calc_single_byte(addrbyte, address & 0x7f);
         /* no second byte => error detection byte = addressbyte */
@@ -1016,7 +1020,7 @@ static void calc_address_stream(char *addrstream, char *addrerrbyte,
     strcat(addrstream, "0");
     strcat(addrstream, addrbyte);
     strcat(addrstream, "0");
-    if (mode == 2) {
+    if (mode == 2) { // Torsten Vogt 2012-11-10
         strcat(addrstream, addrbyte2);
         strcat(addrstream, "0");
     }
@@ -1060,12 +1064,15 @@ int comp_nmra_multi_func(bus_t busnumber, int address, int direction,
                "dir:%d speed:%d nspeeds:%d nfunc:%d",
                mode, address, direction, speed, nspeed, nfuncs);
 
-    adr = address;
+    adr = address; // adr is an additional identifier, only used to store
+                   // the pakets in the internal queue!!!!!
 
+    // Torsten Vogt, 2012-11-10
     if (mode == 2) {
         adr += ADDR14BIT_OFFSET;
     }
     /* no special error handling, it's job of the clients */
+    // Torsten Vogt, 2012-11-10
     if (address < 1 || address > 10239 || direction < 0 || direction > 1 ||
         speed < 0 || speed > (nspeed + 1) || (address > 127 && mode == 1))
         return 1;
@@ -1073,21 +1080,25 @@ int comp_nmra_multi_func(bus_t busnumber, int address, int direction,
     if (speed > 127) {
         speed = 127;
     }
+
     calc_address_stream(addrstream, addrerrbyte, address, mode);
+
     if (speed < 2 || nspeed < 15) {
         /* commands for stop and emergency stop are identical for
            14 and 28 speed steps. All decoders supporting 128
            speed steps also have to support 14/28 speed step commands
            ==> results in shorter refresh cycle if there are many 128
-               speed steps decoders with speed=0, and a slightly faster 
-               emergency stop for these decoders.
-        */
+           speed steps decoders with speed=0, and a slightly faster 
+           emergency stop for these decoders.
+         */
         calc_baseline_speed_byte(spdrbyte, direction, speed, func);
-    } else {
+    }
+    else {
         if (nspeed > 29) {
             calc_128spst_adv_op_bytes(spdrbyte, spdrbyte2, direction,
                                       speed);
-        } else {
+        }
+        else {
             calc_28spst_speed_byte(spdrbyte, direction, speed);
         }
     }
@@ -1113,7 +1124,8 @@ int comp_nmra_multi_func(bus_t busnumber, int address, int direction,
                                             packetstream, false);
         jj = translateBitstream2Packetstream(busnumber, bitstream2,
                                              packetstream2, false);
-    } else {
+    }
+    else {
         j = translateBitstream2Packetstream(busnumber, bitstream,
                                             packetstream, false);
         packetstream2 = packetstream;
@@ -1160,13 +1172,17 @@ int protocol_nmra_sm_write_cvbyte_pom(bus_t busnumber, int address, int cv,
                "command for PoM %d received \naddr:%d CV:%d value:%d",
                mode, address, cv, value);
     cv -= 1;
-    /* do not allow to change the address on the main ==> cv 1 is disabled */
-    if (address < 1 || address > 10239 || cv < 1 || cv > 1023 ||
+
+    if (address < 1 || address > 10239 || cv < 0 || cv > 1023 ||
         value < 0 || value > 255 || (address > 127 && mode == 1))
         return 1;
 
-    adr = address;
+    adr = address; // adr is an additional identifier, only used to store
+                   // the pakets in the internal queue!!!!!
+
+    // Torsten Vogt, 2012-11-10
     if (mode == 2) {
+    // if (mode > 2 && mode < 5) {
         adr += ADDR14BIT_OFFSET;
     }
 
@@ -1220,8 +1236,12 @@ int protocol_nmra_sm_write_cvbit_pom(bus_t busnumber, int address, int cv,
         || (address > 127 && mode == 1))
         return 1;
 
-    adr = address;
+    adr = address; // adr is an additional identifier, only used to store
+                   // the pakets in the internal queue!!!!!
+
+    // Torsten Vogt, 2012-11-10
     if (mode == 2) {
+    // if (mode > 2 && mode < 5) {
         adr += ADDR14BIT_OFFSET;
     }
 
@@ -1320,7 +1340,7 @@ static int waitUARTempty_scanACK(bus_t busnumber)
         if (scanACK(busnumber))
             ack = 1;            /* scan ACK */
         /* prevent endless loop in case somone turned the power on */
-        if (buses[busnumber].power_state) {
+        if (buses[busnumber].power_state == 1) {
             clock_t now = clock();
             float diff = (now - start);
             diff /= CLOCKS_PER_SEC;
@@ -1366,7 +1386,11 @@ static int waitUARTempty_scanACK(bus_t busnumber)
 */
 static int handleACK(bus_t busnumber, int ack)
 {
-    usleep(5000);
+    if (usleep(5000) == -1) {
+        syslog_bus(busnumber, DBG_ERROR,
+                   "usleep() failed: %s (errno = %d)\n",
+                   strerror(errno), errno);
+    }
     /* ack not supported */
     if ((ack == 1) && (scanACK(busnumber) == 1)) {
         return 0;
