@@ -1,13 +1,13 @@
-/* $Id: loconet.h 983 2007-12-16 09:06:42Z gscholz $ */
+/* $Id: loconet.h 1431 2009-12-27 18:33:49Z mtrute $ */
 
 #ifndef _LOCONET_H
 #define _LOCONET_H
 
 #include <libxml/tree.h> /*xmlDocPtr, xmlNodePtr*/
 
-
 #define LN_FLAG_ECHO 1     /* loconet interface sends commands back to rs232 */
 #define LN_FLAG_MS100 2    /* MS100 compatible settings (implicit no echo)   */
+#define LN_FLAG_GETTIME 4  /* update the internal TIME device from loconet   */
 
 typedef struct _LOCONET_DATA {
     int number_fb;              /* used internally */
@@ -20,8 +20,10 @@ typedef struct _LOCONET_DATA {
     unsigned int sent_packets;  /* statistics */
     unsigned int recv_packets;
 
-    unsigned char ln_msglen;       /* length of last message */
-    unsigned char ln_message[128]; /* last message sent */
+    int ibufferin;       /* pointer for incoming characters */
+    int ibufferout;      /* pointer to read from the buffer */
+    unsigned char ibuffer[256]; /* input buffer for loconet packets */
+    unsigned int  slotmap[128];    /* slot to decoder address mapping */
 } LOCONET_DATA;
 
 int readConfig_LOCONET(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber);
