@@ -1,4 +1,4 @@
-/* cvs: $Id: m605x.h 983 2007-12-16 09:06:42Z gscholz $             */
+/* cvs: $Id: m605x.h 1720 2016-03-20 12:38:21Z gscholz $             */
 
 /*
  * Vorliegende Software unterliegt der General Public License,
@@ -29,8 +29,8 @@ int readconfig_m605x(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber);
 
 int init_line6051(bus_t bus);
 int init_bus_M6051(bus_t bus);
-int init_gl_M6051(gl_state_t *gl);
-int init_ga_M6051(ga_state_t *ga);
+int init_gl_M6051(gl_data_t *gl);
+int init_ga_M6051(ga_data_t *ga);
 int getDescription_M6051(char *reply);
 void *thr_sendrec_M6051(void *);
 
