@@ -6,6 +6,7 @@
 
 #include <errno.h>
 #include <string.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 
 #include "config-srcpd.h"
@@ -39,11 +40,10 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
     if (bus_has_devicegroup(bus, DG_GL)
         && strncasecmp(device, "GL", 2) == 0) {
         long laddr, direction, speed, maxspeed, f[29];
-        int func, i, anzparms;
-        func = 0;
+        int func = 0;
         /* We could provide a maximum of 32 on/off functions,
            but for now 28+1 will be good enough */
-        anzparms = sscanf(parameter, "%ld %ld %ld %ld %ld %ld %ld %ld "
+        int nparms = sscanf(parameter, "%ld %ld %ld %ld %ld %ld %ld %ld "
                           "%ld %ld %ld %ld %ld %ld %ld %ld %ld "
                           "%ld %ld %ld %ld %ld %ld %ld %ld "
                           "%ld %ld %ld %ld %ld %ld %ld %ld",
@@ -53,11 +53,11 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
                           &f[14], &f[15], &f[16], &f[17], &f[18], &f[19],
                           &f[20], &f[21], &f[22], &f[23], &f[24], &f[25],
                           &f[26], &f[27], &f[28]);
-        for (i = 0; i < anzparms - 4; i++) {
+        for (int i = 0; i < nparms - 4; i++) {
             func += (f[i] ? 1 : 0) << i;
         }
-        if (anzparms >= 4) {
-            sessionid_t lockid;
+        if (nparms >= 4) {
+            sessionid_t lockid = 0;
             /* Only if not locked or emergency stop !! */
             cacheGetLockGL(bus, laddr, &lockid);
             if (lockid == 0 || lockid == sessionid || direction == 2) {
@@ -79,11 +79,9 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
              && strncasecmp(device, "GA", 2) == 0) {
         long gaddr, port, aktion, delay;
         sessionid_t lockid;
-        int anzparms;
-        anzparms =
-            sscanf(parameter, "%ld %ld %ld %ld", &gaddr, &port, &aktion,
-                   &delay);
-        if (anzparms >= 4) {
+        int nparms = sscanf(parameter, "%ld %ld %ld %ld", &gaddr, &port,
+                &aktion, &delay);
+        if (nparms >= 4) {
             /* Port 0,1; Action 0,1 */
             /* Only if not locked!! */
             getlockGA(bus, gaddr, &lockid);
@@ -104,9 +102,8 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
     else if (bus_has_devicegroup(bus, DG_FB)
              && strncasecmp(device, "FB", 2) == 0) {
         long fbport, value;
-        int anzparms;
-        anzparms = sscanf(parameter, "%ld %ld", &fbport, &value);
-        if (anzparms >= 2) {
+        int nparms = sscanf(parameter, "%ld %ld", &fbport, &value);
+        if (nparms >= 2) {
             if (setorcheck == 1)
                 rc = setFB(bus, fbport, value);
         }
@@ -116,13 +113,11 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
     else if (bus_has_devicegroup(bus, DG_GM)
              && strncasecmp(device, "GM", 2) == 0) {
         sessionid_t sendto, replyto;
-        int result;
         char msg[MAXSRCPLINELEN];
 
         memset(msg, 0, sizeof(msg));
-        /*TODO: scan also message type */
-        result =
-            sscanf(parameter, "%lu %lu %990c", &sendto, &replyto, msg);
+        int result = sscanf(parameter, "%lu %lu %990c", &sendto, &replyto,
+                msg);
         if (result < 3)
             rc = SRCP_LISTTOOSHORT;
         else
@@ -134,10 +129,9 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
              && strncasecmp(device, "SM", 2) == 0) {
         long addr, value1, value2, value3;
         int type;
-        int result;
         char ctype[MAXSRCPLINELEN];
 
-        result = sscanf(parameter, "%ld %s %ld %ld %ld", &addr, ctype,
+        int result = sscanf(parameter, "%ld %s %ld %ld %ld", &addr, ctype,
                         &value1, &value2, &value3);
         if (result < 4)
             rc = SRCP_LISTTOOSHORT;
@@ -170,8 +164,8 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_TIME)
              && strncasecmp(device, "TIME", 4) == 0) {
-        long d, h, m, s, nelem;
-        nelem = sscanf(parameter, "%ld %ld %ld %ld", &d, &h, &m, &s);
+        long d, h, m, s;
+        int nelem = sscanf(parameter, "%ld %ld %ld %ld", &d, &h, &m, &s);
         if (nelem >= 4) {
             rc = SRCP_OK;
             if (setorcheck == 1)
@@ -211,10 +205,9 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_POWER)
              && strncasecmp(device, "POWER", 5) == 0) {
-        int nelem;
         char state[5], msg[256];
         memset(msg, 0, sizeof(msg));
-        nelem = sscanf(parameter, "%3s %100c", state, msg);
+        int nelem = sscanf(parameter, "%3s %100c", state, msg);
         if (nelem >= 1) {
             rc = SRCP_WRONGVALUE;
             if (strncasecmp(state, "OFF", 3) == 0) {
@@ -241,16 +234,16 @@ static int handle_setcheck(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * SET
  */
-int handleSET(sessionid_t sessionid, bus_t bus, char *device,
+static int handleSET(sessionid_t sessionid, bus_t bus, char *device,
               char *parameter, char *reply)
 {
     return handle_setcheck(sessionid, bus, device, parameter, reply, 1);
 }
 
 /***
- * CHECK -- like SET but no command must be sent
+ * CHECK -- like SET but no command must be send
  */
-int handleCHECK(sessionid_t sessionid, bus_t bus, char *device,
+static int handleCHECK(sessionid_t sessionid, bus_t bus, char *device,
                 char *parameter, char *reply)
 {
     return handle_setcheck(sessionid, bus, device, parameter, reply, 0);
@@ -259,8 +252,8 @@ int handleCHECK(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * GET
  */
-int handleGET(sessionid_t sessionid, bus_t bus, char *device,
-              char *parameter, char *reply)
+static int handleGET(sessionid_t sessionid, bus_t bus, char *device,
+              char *parameter, char *reply, size_t length)
 {
     struct timeval akt_time;
     int rc = SRCP_UNSUPPORTEDDEVICEGROUP;
@@ -269,18 +262,18 @@ int handleGET(sessionid_t sessionid, bus_t bus, char *device,
 
     if (bus_has_devicegroup(bus, DG_FB)
         && strncasecmp(device, "FB", 2) == 0) {
-        long int nelem, port;
-        nelem = sscanf(parameter, "%ld", &port);
+        long int port;
+        int nelem = sscanf(parameter, "%ld", &port);
         if (nelem >= 1)
-            rc = infoFB(bus, port, reply);
+            rc = infoFB(bus, port, reply, length);
         else
             rc = srcp_fmt_msg(SRCP_LISTTOOSHORT, reply, akt_time);
     }
 
     else if (bus_has_devicegroup(bus, DG_GL)
              && strncasecmp(device, "GL", 2) == 0) {
-        long nelem, addr;
-        nelem = sscanf(parameter, "%ld", &addr);
+        long addr;
+        int nelem = sscanf(parameter, "%ld", &addr);
         if (nelem >= 1)
             rc = cacheInfoGL(bus, addr, reply);
         else
@@ -289,8 +282,8 @@ int handleGET(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_GA)
              && strncasecmp(device, "GA", 2) == 0) {
-        long addr, port, nelem;
-        nelem = sscanf(parameter, "%ld %ld", &addr, &port);
+        long addr, port;
+        int nelem = sscanf(parameter, "%ld %ld", &addr, &port);
         switch (nelem) {
             case 0:
             case 1:
@@ -307,11 +300,10 @@ int handleGET(sessionid_t sessionid, bus_t bus, char *device,
     else if (bus_has_devicegroup(bus, DG_SM)
              && strncasecmp(device, "SM", 2) == 0) {
         long addr, value1, value2;
-        int nelem;
         int type;
         char ctype[MAXSRCPLINELEN];
 
-        nelem = sscanf(parameter, "%ld %s %ld %ld", &addr, ctype, &value1,
+        int nelem = sscanf(parameter, "%ld %s %ld %ld", &addr, ctype, &value1,
                        &value2);
         if (nelem < 3) {
             rc = SRCP_LISTTOOSHORT;
@@ -367,7 +359,7 @@ int handleGET(sessionid_t sessionid, bus_t bus, char *device,
 
         /* there are two descriptions */
         long int addr;
-        char devgrp[10];
+        char devgrp[11];
         int nelem = 0;
         if (strlen(parameter) > 0)
             nelem = sscanf(parameter, "%10s %ld", devgrp, &addr);
@@ -428,8 +420,8 @@ int handleGET(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * WAIT
  */
-int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
-               char *parameter, char *reply)
+static int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
+               char *parameter, char *reply, size_t length)
 {
     struct timeval time;
     int rc = SRCP_UNSUPPORTEDDEVICEGROUP;
@@ -439,22 +431,26 @@ int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
     /* check, if bus has FB's */
     if (bus_has_devicegroup(bus, DG_FB)
         && strncasecmp(device, "FB", 2) == 0) {
-        long int port, timeout, nelem;
+        long int port, timeout;
         int value, waitvalue;
-        nelem =
-            sscanf(parameter, "%ld %d %ld", &port, &waitvalue, &timeout);
+        int nelem = sscanf(parameter, "%ld %d %ld", &port, &waitvalue,
+                &timeout);
         syslog_bus(bus, DBG_INFO, "wait: %d %d %d", port, waitvalue,
                    timeout);
         if (nelem >= 3) {
             if (getFB(bus, port, &time, &value) == SRCP_OK
                 && value == waitvalue) {
-                rc = infoFB(bus, port, reply);
+                rc = infoFB(bus, port, reply, length);
             }
             else {
                 /* we exactly wait for 1/20 seconds */
                 timeout *= 20;
                 do {
-                    usleep(50000);
+                    if (usleep(50000) == -1) {
+                        syslog_bus(bus, DBG_ERROR,
+                                   "usleep() failed: %s (errno = %d)\n",
+                                   strerror(errno), errno);
+                    }
                     getFB(bus, port, &time, &value);
                     timeout--;
                 }
@@ -465,7 +461,7 @@ int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
                     rc = srcp_fmt_msg(SRCP_TIMEOUT, reply, time);
                 }
                 else {
-                    rc = infoFB(bus, port, reply);
+                    rc = infoFB(bus, port, reply, length);
                 }
             }
         }
@@ -475,15 +471,15 @@ int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
     }
 
     else if (bus_has_devicegroup(bus, DG_TIME)
-            && strncasecmp(device, "TIME", 4) == 0) {
+             && strncasecmp(device, "TIME", 4) == 0) {
         long d, h, m, s;
-        int nelem;
-        nelem = sscanf(parameter, "%ld %ld %ld %ld", &d, &h, &m, &s);
+        int nelem = sscanf(parameter, "%ld %ld %ld %ld", &d, &h, &m, &s);
         if (nelem >= 4) {
-            vtime_t vt;
-            getTIME(&vt);
-            if (vt.ratio_x != 0 && vt.ratio_y != 0) {
+            if (time_is_available()) {
+                /*FIXME: race condition */
                 bool mustwait;
+                vtime_t vt;
+                getTIME(&vt);
                 do {
                     mustwait = (((d * 24 + h) * 60 + m) * 60 + s) >=
                         (((vt.day * 24 + vt.hour) * 60 +
@@ -491,7 +487,11 @@ int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
 
                     /* wait 10ms real time.. */
                     if (mustwait) {
-                        usleep(10000);
+                        if (usleep(10000) == -1) {
+                            syslog_bus(bus, DBG_ERROR,
+                                       "usleep() failed: %s (errno = %d)\n",
+                                       strerror(errno), errno);
+                        }
                         getTIME(&vt);
                     }
                 }
@@ -512,7 +512,7 @@ int handleWAIT(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * VERIFY
  */
-int handleVERIFY(sessionid_t sessionid, bus_t bus, char *device,
+static int handleVERIFY(sessionid_t sessionid, bus_t bus, char *device,
                  char *parameter, char *reply)
 {
     int rc = SRCP_UNSUPPORTEDOPERATION;
@@ -524,10 +524,9 @@ int handleVERIFY(sessionid_t sessionid, bus_t bus, char *device,
         && strncasecmp(device, "SM", 2) == 0) {
         long addr, value1, value2, value3;
         int type;
-        int result;
         char ctype[MAXSRCPLINELEN];
 
-        result = sscanf(parameter, "%ld %s %ld %ld %ld", &addr, ctype,
+        int result = sscanf(parameter, "%ld %s %ld %ld %ld", &addr, ctype,
                         &value1, &value2, &value3);
         if (result < 4)
             rc = SRCP_LISTTOOSHORT;
@@ -567,7 +566,7 @@ int handleVERIFY(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * TERM
  * negative return code (rc) will terminate current session! */
-int handleTERM(sessionid_t sessionid, bus_t bus, char *device,
+static int handleTERM(sessionid_t sessionid, bus_t bus, char *device,
                char *parameter, char *reply)
 {
     struct timeval akt_time;
@@ -583,16 +582,39 @@ int handleTERM(sessionid_t sessionid, bus_t bus, char *device,
         if (nelem == 1) {
             sessionid_t lockid;
             cacheGetLockGL(bus, addr, &lockid);
-            if (lockid == 0 || lockid == sessionid) {
-                rc = cacheUnlockGL(bus, addr, sessionid);
+            if (lockid == 0) {
                 rc = cacheTermGL(bus, addr);
             }
+            else if (lockid == sessionid) {
+                if (SRCP_OK == cacheUnlockGL(bus, addr, sessionid))
+                    rc = cacheTermGL(bus, addr);
+            }
+            else {
+                rc = SRCP_DEVICELOCKED;
+            }
+        }
+    }
+    else if (bus_has_devicegroup(bus, DG_GA)
+             && strncasecmp(device, "GA", 2) == 0) {
+        long int addr = 0;
+        int nelem = 0;
+        if (strlen(parameter) > 0)
+            nelem = sscanf(parameter, "%ld", &addr);
+        if (nelem == 1) {
+            sessionid_t lockid;
+            getlockGA(bus, addr, &lockid);
+            if (lockid == 0) {
+                rc = termGA(bus, addr);
+            }
+            else if (lockid == sessionid) {
+                if (SRCP_OK == unlockGA(bus, addr, sessionid))
+                    rc = termGA(bus, addr);
+            }
             else {
                 rc = SRCP_DEVICELOCKED;
             }
         }
     }
-
     else if (bus_has_devicegroup(bus, DG_LOCK)
              && strncasecmp(device, "LOCK", 4) == 0) {
         long int addr;
@@ -642,6 +664,11 @@ int handleTERM(sessionid_t sessionid, bus_t bus, char *device,
         rc = infoSM(bus, TERM, 0, -1, 0, 0, 0, reply);
     }
 
+    else if (bus_has_devicegroup(bus, DG_TIME)
+             && strncasecmp(device, "TIME", 4) == 0) {
+        rc = termTIME();
+    }
+
     gettimeofday(&akt_time, NULL);
     srcp_fmt_msg(abs(rc), reply, akt_time);
     return rc;
@@ -650,7 +677,7 @@ int handleTERM(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * INIT
  */
-int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
+static int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
                char *parameter, char *reply)
 {
     struct timeval time;
@@ -659,13 +686,20 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
     /*INIT <bus> GL "<addr> <protocol> <optional further parameters>" */
     if (bus_has_devicegroup(bus, DG_GL)
         && strncasecmp(device, "GL", 2) == 0) {
-        long addr, protversion, n_fs, n_func, nelem;
+        long addr, protversion, n_fs, n_func;
         char prot;
-        nelem =
-            sscanf(parameter, "%ld %c %ld %ld %ld", &addr, &prot,
+        int nelem = sscanf(parameter, "%ld %c %ld %ld %ld", &addr, &prot,
                    &protversion, &n_fs, &n_func);
         if (nelem >= 5) {
-            rc = cacheInitGL(bus, addr, prot, protversion, n_fs, n_func);
+            sessionid_t lockid = 0;
+            /* Only if not locked !! */
+            cacheGetLockGL(bus, addr, &lockid);
+            if (lockid == 0 || lockid == sessionid) {
+                rc = cacheInitGL(bus, addr, prot, protversion, n_fs, n_func);
+            }
+            else {
+                rc = SRCP_DEVICELOCKED;
+            }
         }
         else {
             rc = SRCP_LISTTOOSHORT;
@@ -674,9 +708,9 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_GA)
              && strncasecmp(device, "GA", 2) == 0) {
-        long addr, nelem;
+        long addr;
         char prot;
-        nelem = sscanf(parameter, "%ld %c", &addr, &prot);
+        int nelem = sscanf(parameter, "%ld %c", &addr, &prot);
         if (nelem >= 2) {
             rc = initGA(bus, addr, prot);
         }
@@ -687,9 +721,9 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_FB)
              && strncasecmp(device, "FB", 2) == 0) {
-        long addr, index, nelem;
+        long addr, index;
         char prot;
-        nelem = sscanf(parameter, "%ld %c %ld", &addr, &prot, &index);
+        int nelem = sscanf(parameter, "%ld %c %ld", &addr, &prot, &index);
         if (nelem >= 3) {
             rc = initFB(bus, addr, prot, index);
         }
@@ -705,9 +739,8 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
 
     else if (bus_has_devicegroup(bus, DG_TIME)
              && strncasecmp(device, "TIME", 4) == 0) {
-        int nelem;
         long rx, ry;
-        nelem = sscanf(parameter, "%ld %ld", &rx, &ry);
+        int nelem = sscanf(parameter, "%ld %ld", &rx, &ry);
         if (nelem >= 2) {
             rc = initTIME(rx, ry);      /* checks also values! */
         }
@@ -719,14 +752,13 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
     /* INIT <bus> SM "<protocol>" */
     else if (bus_has_devicegroup(bus, DG_SM)
              && strncasecmp(device, "SM", 2) == 0) {
-        int result;
         char protocol[MAXSRCPLINELEN];
 
-        result = sscanf(parameter, "%s", protocol);
+        int result = sscanf(parameter, "%s", protocol);
         if (result < 1)
             rc = SRCP_LISTTOOSHORT;
         else if (strncasecmp(protocol, "NMRA", 4) == 0)
-            rc = infoSM(bus, INIT, 0, -1, 0, 0, 0, reply);
+            rc = infoSM(bus, INIT, 0, -1, NMRA, 0, 0, reply);
         else
             rc = SRCP_WRONGVALUE;
     }
@@ -739,11 +771,18 @@ int handleINIT(sessionid_t sessionid, bus_t bus, char *device,
 /**
  * RESET
  */
-int handleRESET(sessionid_t sessionid, bus_t bus, char *device,
+static int handleRESET(sessionid_t sessionid, bus_t bus, char *device,
                 char *parameter, char *reply)
 {
     struct timeval time;
     int rc = SRCP_UNSUPPORTEDOPERATION;
+    *reply = 0x00;
+
+    if (bus_has_devicegroup(bus, DG_SERVER)
+             && strncasecmp(device, "SERVER", 6) == 0) {
+        rc = SRCP_OK;
+        server_reset();
+    }
 
     gettimeofday(&time, NULL);
     srcp_fmt_msg(rc, reply, time);
@@ -765,17 +804,16 @@ int doCmdClient(session_node_t * sn)
     char devicegroup[MAXSRCPLINELEN];
     char parameter[MAXSRCPLINELEN];
     bus_t bus;
-    long int rc, nelem;
+    int rc;
     struct timeval akt_time;
-    ssize_t result;
 
     syslog_session(sn->session, DBG_INFO, "Command mode starting.");
 
-    while (1) {
+    while (true) {
         pthread_testcancel();
         memset(line, 0, sizeof(line));
 
-        result = socket_readline(sn->socket, line, sizeof(line) - 1);
+        ssize_t result = socket_readline(sn->socket, line, sizeof(line) - 1);
 
         /* client terminated connection */
         if (0 == result) {
@@ -791,16 +829,17 @@ int doCmdClient(session_node_t * sn)
             break;
         }
 
-        /*remove terminating line break*/
+        /*remove terminating line break */
         size_t linelen = strlen(line);
         if (linelen > 1 && (line[linelen - 1] == '\n'))
             line[linelen - 1] = '\0';
 
         memset(command, 0, sizeof(command));
+        memset(cbus, 0, sizeof(cbus));
         memset(devicegroup, 0, sizeof(devicegroup));
         memset(parameter, 0, sizeof(parameter));
         memset(reply, 0, sizeof(reply));
-        nelem = sscanf(line, "%s %s %s %1000c", command, cbus,
+        int nelem = sscanf(line, "%s %s %s %1000c", command, cbus,
                        devicegroup, parameter);
         bus = atoi(cbus);
         reply[0] = 0x00;
@@ -812,13 +851,17 @@ int doCmdClient(session_node_t * sn)
                     rc = handleSET(sn->session, bus, devicegroup,
                                    parameter, reply);
                 }
+                else if (strncasecmp(command, "CHECK", 5) == 0) {
+                    rc = handleCHECK(sn->session, bus, devicegroup,
+                                   parameter, reply);
+                }
                 else if (strncasecmp(command, "GET", 3) == 0) {
                     rc = handleGET(sn->session, bus, devicegroup,
-                                   parameter, reply);
+                                   parameter, reply, sizeof(reply));
                 }
                 else if (strncasecmp(command, "WAIT", 4) == 0) {
                     rc = handleWAIT(sn->session, bus, devicegroup,
-                                    parameter, reply);
+                                    parameter, reply, sizeof(reply));
                 }
                 else if (strncasecmp(command, "INIT", 4) == 0) {
                     rc = handleINIT(sn->session, bus, devicegroup,
