@@ -1,4 +1,4 @@
-/* $Id: srcp-descr.c 1016 2007-12-27 15:58:33Z gscholz $ */
+/* $Id: srcp-descr.c 1725 2016-03-20 17:06:01Z gscholz $ */
 
 /*
  * Vorliegende Software unterliegt der General Public License,
@@ -11,9 +11,8 @@
 #include "srcp-error.h"
 #include "config-srcpd.h"
 
-int startup_DESCRIPTION(void)
+void startup_DESCRIPTION()
 {
-    return 0;
 }
 
 int describeBus(bus_t bus, char *reply)
