@@ -1,4 +1,4 @@
-/* $Id: srcp-fb.h 1044 2007-12-30 10:42:52Z gscholz $ */
+/* $Id: srcp-fb.h 1725 2016-03-20 17:06:01Z gscholz $ */
 
 /*
  * Vorliegende Software unterliegt der General Public License,
@@ -32,7 +32,7 @@ typedef struct _RESET_FB
   struct timeval timestamp;
 } fb_reset_t;
 
-int startup_FB(void);
+void startup_FB();
 int init_FB(bus_t bus, int number);
 int get_number_fb(bus_t bus);
 int initFB(bus_t busnumber, int addr, const char protocol, int index);
@@ -41,7 +41,7 @@ int getFB(bus_t bus, int port, struct timeval *time, int *value);
 int setFB( bus_t bus, int port,  int value );
 int updateFB(bus_t bus, int port, int value);
 int setFBmodul(bus_t bus, int mod, int values);
-int infoFB(bus_t bus, int port, char *msg);
+int infoFB(bus_t bus, int port, char *msg, size_t length);
 int describeFB(bus_t bus, int addr, char *reply);
 void check_reset_fb(bus_t busnumber);
 void set_min_time(bus_t busnumber, int mt);
