@@ -1,4 +1,4 @@
-/* $Id: srcp-gl.c 1740 2016-04-24 12:36:36Z gscholz $ */
+/* $Id: srcp-gl.c 1765 2021-03-02 21:43:13Z gscholz $ */
 
 #include <string.h>
 #include <stdio.h>
@@ -29,7 +29,7 @@ static int queue_isfull(bus_t busnumber);
  * isValidGL: checks if a given address could be a valid GL.
  * returns true or false. false, if not all requirements are met.
  */
-int isValidGL(bus_t busnumber, int addr)
+bool isValidGL(bus_t busnumber, int addr)
 {
     /* in bus 0 GL are not allowed */
     /* only num_buses are configured */
@@ -52,8 +52,7 @@ unsigned int getMaxAddrGL(bus_t busnumber)
     if (busnumber <= num_buses) {
         return gl[busnumber].glcount;
     }
-    else
-        return 0;
+    return 0;
 }
 
 /* there are decoders for 14, 27, 28 and 128 speed steps */
@@ -89,9 +88,7 @@ bool isInitializedGL(bus_t busnumber, int addr)
     if (isValidGL(busnumber, addr)) {
         return (gl[busnumber].glstate[addr].state == glsActive);
     }
-    else {
-        return false;
-    }
+    return false;
 }
 
 /* Take new locomotive data and make some checks.
@@ -101,57 +98,55 @@ bool isInitializedGL(bus_t busnumber, int addr)
 int enqueueGL(bus_t busnumber, int addr, int dir, int speed, int maxspeed,
               int f)
 {
+    if (!isValidGL(busnumber, addr))
+        return SRCP_WRONGVALUE;
+
     int result;
     struct timeval now;
 
-    if (isValidGL(busnumber, addr)) {
-        if (!isInitializedGL(busnumber, addr)) {
-            cacheInitGL(busnumber, addr, 'P', 1, 14, 1);
-            syslog_bus(busnumber, DBG_WARN, "GL default init for %d-%d",
-                       busnumber, addr);
-        }
-        if (queue_isfull(busnumber)) {
-            syslog_bus(busnumber, DBG_WARN, "GL command queue full");
-            return SRCP_TEMPORARILYPROHIBITED;
-        }
-
-        result = pthread_mutex_lock(&queue_mutex[busnumber]);
-        if (result != 0) {
-            syslog_bus(busnumber, DBG_ERROR,
-                       "pthread_mutex_lock() failed: %s (errno = %d).",
-                       strerror(result), result);
-        }
-
-        /* copy all known (INIT) values to queue */
-        queue[busnumber][in[busnumber]] = gl[busnumber].glstate[addr];
-
-        /* update remaining new variables */
-        queue[busnumber][in[busnumber]].speed =
-            calcspeed(speed, maxspeed, gl[busnumber].glstate[addr].n_fs);
-
-        queue[busnumber][in[busnumber]].direction = dir;
-        queue[busnumber][in[busnumber]].funcs = f;
-        gettimeofday(&now, NULL);
-        queue[busnumber][in[busnumber]].tv = now;
-        queue[busnumber][in[busnumber]].id = addr;
-        in[busnumber]++;
-        if (in[busnumber] == QUEUELEN)
-            in[busnumber] = 0;
-
-        result = pthread_mutex_unlock(&queue_mutex[busnumber]);
-        if (result != 0) {
-            syslog_bus(busnumber, DBG_ERROR,
-                       "pthread_mutex_unlock() failed: %s (errno = %d).",
-                       strerror(result), result);
-        }
-
-        /* Restart thread to send GL command */
-        resume_bus_thread(busnumber);
-        return SRCP_OK;
+    if (!isInitializedGL(busnumber, addr)) {
+        cacheInitGL(busnumber, addr, 'P', 1, 14, 1);
+        syslog_bus(busnumber, DBG_WARN, "GL default init for %d-%d",
+                busnumber, addr);
     }
-    else {
-        return SRCP_WRONGVALUE;
+    if (queue_isfull(busnumber)) {
+        syslog_bus(busnumber, DBG_WARN, "GL command queue full");
+        return SRCP_TEMPORARILYPROHIBITED;
     }
+
+    result = pthread_mutex_lock(&queue_mutex[busnumber]);
+    if (result != 0) {
+        syslog_bus(busnumber, DBG_ERROR,
+                "pthread_mutex_lock() failed: %s (errno = %d).",
+                strerror(result), result);
+    }
+
+    /* copy all known (INIT) values to queue */
+    queue[busnumber][in[busnumber]] = gl[busnumber].glstate[addr];
+
+    /* update remaining new variables */
+    queue[busnumber][in[busnumber]].speed =
+        calcspeed(speed, maxspeed, gl[busnumber].glstate[addr].n_fs);
+
+    queue[busnumber][in[busnumber]].direction = dir;
+    queue[busnumber][in[busnumber]].funcs = f;
+    gettimeofday(&now, NULL);
+    queue[busnumber][in[busnumber]].tv = now;
+    queue[busnumber][in[busnumber]].id = addr;
+    in[busnumber]++;
+    if (in[busnumber] == QUEUELEN)
+        in[busnumber] = 0;
+
+    result = pthread_mutex_unlock(&queue_mutex[busnumber]);
+    if (result != 0) {
+        syslog_bus(busnumber, DBG_ERROR,
+                "pthread_mutex_unlock() failed: %s (errno = %d).",
+                strerror(result), result);
+    }
+
+    /* Restart thread to send GL command */
+    resume_bus_thread(busnumber);
+    return SRCP_OK;
 }
 
 int queue_GL_isempty(bus_t busnumber)
@@ -188,13 +183,11 @@ int dequeueNextGL(bus_t busnumber, gl_data_t *gld)
 
 int cacheGetGL(bus_t busnumber, int addr, gl_data_t *gld)
 {
-    if (isInitializedGL(busnumber, addr)) {
-        *gld = gl[busnumber].glstate[addr];
-        return SRCP_OK;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_NODATA;
-    }
+
+    *gld = gl[busnumber].glstate[addr];
+    return SRCP_OK;
 }
 
 /**
@@ -205,62 +198,53 @@ int cacheGetGL(bus_t busnumber, int addr, gl_data_t *gld)
 */
 int cacheSetGL(bus_t busnumber, int addr, gl_data_t l)
 {
-    if (isValidGL(busnumber, addr)) {
-        char msg[1000];
-        gl[busnumber].glstate[addr].direction = l.direction;
-        gl[busnumber].glstate[addr].speed = l.speed;
-        gl[busnumber].glstate[addr].funcs = l.funcs;
-        gl[busnumber].glstate[addr].n_fs = l.n_fs;
-        gl[busnumber].glstate[addr].n_func = l.n_func;
-        gettimeofday(&gl[busnumber].glstate[addr].tv, NULL);
-        if (gl[busnumber].glstate[addr].state == glsTerm) {
-            snprintf(msg, sizeof(msg), "%lu.%.3lu 102 INFO %ld GL %d\n",
-                     gl[busnumber].glstate[addr].tv.tv_sec,
-                     gl[busnumber].glstate[addr].tv.tv_usec / 1000,
-                     busnumber, addr);
-            bzero(&gl[busnumber].glstate[addr], sizeof(gl_data_t));
-        }
-        else {
-            cacheInfoGL(busnumber, addr, msg);
-        }
-        enqueueInfoMessage(msg);
-        return SRCP_OK;
+    if (!isValidGL(busnumber, addr))
+        return SRCP_WRONGVALUE;
+
+    char msg[1000];
+    gl[busnumber].glstate[addr] = l;
+    gettimeofday(&gl[busnumber].glstate[addr].tv, NULL);
+    if (gl[busnumber].glstate[addr].state == glsTerm) {
+        snprintf(msg, sizeof(msg), "%lu.%.3lu 102 INFO %ld GL %d\n",
+                gl[busnumber].glstate[addr].tv.tv_sec,
+                gl[busnumber].glstate[addr].tv.tv_usec / 1000,
+                busnumber, addr);
+        bzero(&gl[busnumber].glstate[addr], sizeof(gl_data_t));
     }
     else {
-        return SRCP_WRONGVALUE;
+        cacheInfoGL(busnumber, addr, msg);
     }
+    enqueueInfoMessage(msg);
+    return SRCP_OK;
 }
 
 int cacheInitGL(bus_t busnumber, int addr, const char protocol,
                 int protoversion, int n_fs, int n_func)
 {
-    int rc = SRCP_WRONGVALUE;
-    if (isValidGL(busnumber, addr)) {
-        char msg[1000];
-        gl_data_t tgl;
-        memset(&tgl, 0, sizeof(tgl));
-        rc = bus_supports_protocol(busnumber, protocol);
-        if (rc != SRCP_OK) {
-            return rc;
-        }
-        gettimeofday(&tgl.inittime, NULL);
-        tgl.tv = tgl.inittime;
-        tgl.n_fs = n_fs;
-        tgl.n_func = n_func;
-        tgl.protocolversion = protoversion;
-        tgl.protocol = protocol;
-        tgl.id = addr;
-        if (buses[busnumber].init_gl_func)
-            rc = (*buses[busnumber].init_gl_func) (&tgl);
-        if (rc == SRCP_OK) {
-            gl[busnumber].glstate[addr] = tgl;
-            gl[busnumber].glstate[addr].state = glsActive;
-            cacheDescribeGL(busnumber, addr, msg);
-            enqueueInfoMessage(msg);
-        }
+    if (!isValidGL(busnumber, addr))
+        return SRCP_WRONGVALUE;
+
+    char msg[1000];
+    gl_data_t tgl;
+    memset(&tgl, 0, sizeof(tgl));
+    int rc = bus_supports_protocol(busnumber, protocol);
+    if (rc != SRCP_OK) {
+        return rc;
     }
-    else {
-        rc = SRCP_WRONGVALUE;
+    gettimeofday(&tgl.inittime, NULL);
+    tgl.tv = tgl.inittime;
+    tgl.n_fs = n_fs;
+    tgl.n_func = n_func;
+    tgl.protocolversion = protoversion;
+    tgl.protocol = protocol;
+    tgl.id = addr;
+    if (buses[busnumber].init_gl_func)
+        rc = (*buses[busnumber].init_gl_func) (&tgl);
+    if (rc == SRCP_OK) {
+        gl[busnumber].glstate[addr] = tgl;
+        gl[busnumber].glstate[addr].state = glsActive;
+        cacheDescribeGL(busnumber, addr, msg);
+        enqueueInfoMessage(msg);
     }
     return rc;
 }
@@ -268,36 +252,33 @@ int cacheInitGL(bus_t busnumber, int addr, const char protocol,
 
 int cacheTermGL(bus_t busnumber, int addr)
 {
-    gl_data_t gltmp;
-
-    if (isInitializedGL(busnumber, addr)) {
-        cacheGetGL(busnumber, addr, &gltmp);
-
-        /* first brake if current speed != 0 */
-        if (gltmp.speed != 0) {
-            gltmp.speed = 0;
-            gltmp.state = glsTerm;
-            gltmp.direction = 2;
-            enqueueGL(busnumber, addr, gltmp.direction, gltmp.speed,
-                    gltmp.n_fs, gltmp.funcs);
-            cacheSetGL(busnumber, addr, gltmp);
-        }
-
-        /* second terminate GL */
-        char msg[256];
-        gettimeofday(&gltmp.tv, NULL);
-        snprintf(msg, sizeof(msg), "%lu.%.3lu 102 INFO %ld GL %d\n",
-                gltmp.tv.tv_sec, gltmp.tv.tv_usec / 1000, busnumber, addr);
-        enqueueInfoMessage(msg);
-
-        /* third clear GL data */
-        memset(&gl[busnumber].glstate[addr], 0, sizeof(gl_data_t));
-
-        return SRCP_OK;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_NODATA;
+
+    gl_data_t *gld = &gl[busnumber].glstate[addr];
+
+    /*we are terminating now*/
+    gld->state = glsTerm;
+
+    /* first brake if current speed != 0 */
+    if (gld->speed != 0) {
+        gld->speed = 0;
+        gld->direction = 2;
+        enqueueGL(busnumber, addr, gld->direction, gld->speed,
+                gld->n_fs, gld->funcs);
     }
+
+    /* second terminate GL */
+    char msg[256];
+    gettimeofday(&gld->tv, NULL);
+    snprintf(msg, sizeof(msg), "%lu.%.3lu 102 INFO %ld GL %d\n",
+            gld->tv.tv_sec, gld->tv.tv_usec / 1000, busnumber, addr);
+    enqueueInfoMessage(msg);
+
+    /* third clear GL data */
+    memset(gld, 0, sizeof(gl_data_t));
+
+    return SRCP_OK;
 }
 
 /*
@@ -305,13 +286,11 @@ int cacheTermGL(bus_t busnumber, int addr)
  */
 int resetGL(bus_t busnumber, int addr)
 {
-    if (isInitializedGL(busnumber, addr)) {
-        enqueueGL(busnumber, addr, 0, 0, 1, 0);
-        return SRCP_OK;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_NODATA;
-    }
+
+    enqueueGL(busnumber, addr, 0, 0, 1, 0);
+    return SRCP_OK;
 }
 
 
@@ -325,102 +304,90 @@ void cacheCleanGL(bus_t bus)
 
 int cacheDescribeGL(bus_t busnumber, int addr, char *msg)
 {
-    if (isInitializedGL(busnumber, addr)) {
-        sprintf(msg, "%lu.%.3lu 101 INFO %ld GL %d %c %d %d %d\n",
-                gl[busnumber].glstate[addr].inittime.tv_sec,
-                gl[busnumber].glstate[addr].inittime.tv_usec / 1000,
-                busnumber, addr, gl[busnumber].glstate[addr].protocol,
-                gl[busnumber].glstate[addr].protocolversion,
-                gl[busnumber].glstate[addr].n_fs,
-                gl[busnumber].glstate[addr].n_func);
-        return SRCP_INFO;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr)) {
         strcpy(msg, "");
         return SRCP_NODATA;
     }
+
+    gl_data_t* gld = &gl[busnumber].glstate[addr];
+    sprintf(msg, "%lu.%.3lu 101 INFO %ld GL %d %c %d %d %d\n",
+            gld->inittime.tv_sec,
+            gld->inittime.tv_usec / 1000,
+            busnumber, addr, gld->protocol,
+            gld->protocolversion,
+            gld->n_fs,
+            gld->n_func);
+    return SRCP_INFO;
 }
 
 int cacheInfoGL(bus_t busnumber, int addr, char *msg)
 {
-    char line[MAXSRCPLINELEN];
-
-    /*get address of specified GL data*/
-    gl_data_t* glptr = &gl[busnumber].glstate[addr];
-    if (glptr == NULL)
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_NODATA;
 
-    if (isInitializedGL(busnumber, addr)) {
-        sprintf(msg, "%lu.%.3lu 100 INFO %ld GL %d %d %d %d %d",
-                glptr->tv.tv_sec, glptr->tv.tv_usec / 1000,
-                busnumber, addr, glptr->direction,
-                glptr->speed, glptr->n_fs,
-                (glptr->funcs & 0x01) ? 1 : 0);
+    char line[MAXSRCPLINELEN];
+    gl_data_t* gld = &gl[busnumber].glstate[addr];
 
-        for (int i = 1; i < glptr->n_func; i++) {
-            snprintf(line, sizeof(line), "%s %d", msg,
-                    ((glptr->funcs >> i) & 0x01) ? 1 : 0);
-            strcpy(msg, line);
-        }
-        snprintf(line, sizeof(line), "%s\n", msg);
+    sprintf(msg, "%lu.%.3lu 100 INFO %ld GL %d %d %d %d %d",
+            gld->tv.tv_sec, gld->tv.tv_usec / 1000,
+            busnumber, addr, gld->direction,
+            gld->speed, gld->n_fs,
+            (gld->funcs & 0x01) ? 1 : 0);
+
+    for (int i = 1; i < gld->n_func; i++) {
+        snprintf(line, sizeof(line), "%s %d", msg,
+                ((gld->funcs >> i) & 0x01) ? 1 : 0);
         strcpy(msg, line);
-
-        return SRCP_INFO;
     }
-    return SRCP_NODATA;
+    snprintf(line, sizeof(line), "%s\n", msg);
+    strcpy(msg, line);
+
+    return SRCP_INFO;
 }
 
 /* has to use a semaphore, must be atomized! */
 int cacheLockGL(bus_t busnumber, int addr, long int duration,
                 sessionid_t sessionid)
 {
-    char msg[256];
-
-    if (isInitializedGL(busnumber, addr)) {
-        if (gl[busnumber].locked_by == sessionid
-            || gl[busnumber].locked_by == 0) {
-            gl[busnumber].locked_by = sessionid;
-            gl[busnumber].lockduration = duration;
-            gettimeofday(&gl[busnumber].locktime, NULL);
-            describeLOCKGL(busnumber, addr, msg);
-            enqueueInfoMessage(msg);
-            return SRCP_OK;
-        }
-        else {
-            return SRCP_DEVICELOCKED;
-        }
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_WRONGVALUE;
+
+    char msg[256];
+    gl_data_t *gld = &gl[busnumber].glstate[addr];
+
+    if (gld->locked_by == sessionid || gld->locked_by == 0) {
+        gld->locked_by = sessionid;
+        gld->lockduration = duration;
+        gettimeofday(&gld->locktime, NULL);
+        describeLOCKGL(busnumber, addr, msg);
+        enqueueInfoMessage(msg);
+        return SRCP_OK;
     }
+    return SRCP_DEVICELOCKED;
 }
 
 int cacheGetLockGL(bus_t busnumber, int addr, sessionid_t * session_id)
 {
-    if (isInitializedGL(busnumber, addr)) {
-
-        *session_id = gl[busnumber].locked_by;
-        return SRCP_OK;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_WRONGVALUE;
-    }
+
+    *session_id = gl[busnumber].glstate[addr].locked_by;
+    return SRCP_OK;
 }
 
-int describeLOCKGL(bus_t bus, int addr, char *reply)
+int describeLOCKGL(bus_t busnumber, int addr, char *reply)
 {
-    if (isInitializedGL(bus, addr)) {
-
-        sprintf(reply, "%lu.%.3lu 100 INFO %ld LOCK GL %d %ld %ld\n",
-                gl[bus].locktime.tv_sec,
-                gl[bus].locktime.tv_usec / 1000,
-                bus, addr, gl[bus].lockduration,
-                gl[bus].locked_by);
-        return SRCP_OK;
-    }
-    else {
+    if (!isInitializedGL(busnumber, addr))
         return SRCP_WRONGVALUE;
-    }
+
+    gl_data_t *gld = &gl[busnumber].glstate[addr];
+
+    sprintf(reply, "%lu.%.3lu 100 INFO %ld LOCK GL %d %ld %ld\n",
+            gld->locktime.tv_sec,
+            gld->locktime.tv_usec / 1000,
+            busnumber, addr, gld->lockduration,
+            gld->locked_by);
+    return SRCP_OK;
 }
 
 int cacheUnlockGL(bus_t busnumber, int addr, sessionid_t sessionid)
@@ -428,21 +395,20 @@ int cacheUnlockGL(bus_t busnumber, int addr, sessionid_t sessionid)
     if (!isInitializedGL(busnumber, addr))
         return SRCP_WRONGVALUE;
 
-    if (gl[busnumber].locked_by == 0 ||
-            gl[busnumber].locked_by == sessionid) {
+    gl_data_t *gld = &gl[busnumber].glstate[addr];
+
+    if (gld->locked_by == 0 || gld->locked_by == sessionid) {
         char msg[256];
-        gl[busnumber].locked_by = 0;
-        gettimeofday(&gl[busnumber].locktime, NULL);
+        gld->locked_by = 0;
+        gettimeofday(&gld->locktime, NULL);
         sprintf(msg, "%lu.%.3lu 102 INFO %ld LOCK GL %d\n",
-                gl[busnumber].locktime.tv_sec,
-                gl[busnumber].locktime.tv_usec / 1000,
+                gld->locktime.tv_sec,
+                gld->locktime.tv_usec / 1000,
                 busnumber, addr);
         enqueueInfoMessage(msg);
         return SRCP_OK;
     }
-    else {
-        return SRCP_DEVICELOCKED;
-    }
+    return SRCP_DEVICELOCKED;
 }
 
 /**
@@ -454,7 +420,7 @@ void unlock_gl_bysessionid(sessionid_t sessionid)
     for (bus_t i = 0; i <= num_buses; i++) {
         int number = getMaxAddrGL(i);
         for (int j = 1; j <= number; j++) {
-            if (gl[i].locked_by == sessionid) {
+            if (gl[i].glstate[j].locked_by == sessionid) {
                 cacheUnlockGL(i, j, sessionid);
             }
         }
@@ -469,9 +435,9 @@ void unlock_gl_bytime(void)
     for (bus_t i = 0; i <= num_buses; i++) {
         int number = getMaxAddrGL(i);
         for (int j = 1; j <= number; j++) {
-            if (gl[i].lockduration > 0
-                && gl[i].lockduration-- == 1) {
-                cacheUnlockGL(i, j, gl[i].locked_by);
+            if (gl[i].glstate[j].lockduration > 0
+                && gl[i].glstate[j].lockduration-- == 1) {
+                cacheUnlockGL(i, j, gl[i].glstate[j].locked_by);
             }
         }
     }
@@ -556,10 +522,7 @@ void debugGL(bus_t busnumber, int start, int end)
         syslog_bus(busnumber, DBG_WARN, "direction %d", gld->direction);
         syslog_bus(busnumber, DBG_WARN, "funcs %d", gld->funcs);
         syslog_bus(busnumber, DBG_WARN, "lockduration %ld",
-                   gl->lockduration);
-        syslog_bus(busnumber, DBG_WARN, "locked_by %ld", gl->locked_by);
-        /*  struct timeval tv;
-           struct timeval inittime;
-           struct timeval locktime; */
+                   gld->lockduration);
+        syslog_bus(busnumber, DBG_WARN, "locked_by %ld", gld->locked_by);
     }
 }
