@@ -1,4 +1,4 @@
-/* $Id: srcp-gl.h 1725 2016-03-20 17:06:01Z gscholz $ */
+/* $Id: srcp-gl.h 1764 2021-03-02 21:29:30Z gscholz $ */
 
 #ifndef _SRCP_GL_H
 #define _SRCP_GL_H
@@ -24,15 +24,15 @@ typedef struct {
     unsigned int funcs;       /* Fx..F1, F                           */
     struct timeval tv;        /* Last time of change                 */
     struct timeval inittime;
+    struct timeval locktime;
+    sessionid_t locked_by;    /* locking session */
+    long int lockduration;
 } gl_data_t;
 
 /* GL device group data set */
 typedef struct {
     unsigned int glcount;
     gl_data_t *glstate;
-    struct timeval locktime;
-    long int lockduration;
-    sessionid_t locked_by;
 } gl_t;
 
 void startup_GL();
@@ -40,7 +40,7 @@ void shutdown_GL();
 int init_GL(bus_t busnumber, unsigned int count);
 unsigned int getMaxAddrGL(bus_t busnumber);
 bool isInitializedGL(bus_t busnumber, int addr);
-int isValidGL(bus_t busnumber, int addr);
+bool isValidGL(bus_t busnumber, int addr);
 int enqueueGL(bus_t busnumber, int addr, int dir, int speed,
         int maxspeed, int f);
 int queue_GL_isempty(bus_t busnumber);
