@@ -2,8 +2,8 @@
                           srcp-info.h  -  description
                              -------------------
     begin                : Mon May 20 2002
-    copyright            : (C) 2002 by 
-    email                : 
+    copyright            : (C) 2002 by
+    email                :
  ***************************************************************************/
 
 /***************************************************************************
@@ -21,7 +21,7 @@
 #include "clientservice.h"
 
 int doInfoClient(session_node_t*);
-int startup_INFO(void);
+void startup_INFO();
 int enqueueInfoMessage(char *);
 
 #endif
